@@ -176,10 +176,12 @@ g_unix_connection_receive_fd (GUnixConnection  *connection,
     {
       gint i;
 
-      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           ngettext ("Expecting 1 control message, got %d",
-                                     "Expecting 1 control message, got %d",
-                                     nscm));
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   g_dngettext (NULL,
+                                "Expecting 1 control message, got %d",
+                                "Expecting 1 control message, got %d",
+                                nscm),
+                   nscm);
 
       for (i = 0; i < nscm; i++)
         g_object_unref (scms[i]);
@@ -209,10 +211,12 @@ g_unix_connection_receive_fd (GUnixConnection  *connection,
     {
       gint i;
 
-      g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                           ngettext ("Expecting one fd, but got %d\n",
-                                     "Expecting one fd, but got %d\n",
-                                     nfd));
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   g_dngettext (NULL,
+                                "Expecting one fd, but got %d\n",
+                                "Expecting one fd, but got %d\n",
+                                nfd),
+                   nfd);
 
       for (i = 0; i < nfd; i++)
         close (fds[i]);
@@ -412,8 +416,9 @@ send_credentials_async_thread (GTask         *task,
  * g_unix_connection_send_credentials_async:
  * @connection: A #GUnixConnection.
  * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (scope async): a #GAsyncReadyCallback to call when the request is satisfied
- * @user_data: (closure): the data to pass to callback function
+ * @callback: (scope async): a #GAsyncReadyCallback
+ *   to call when the request is satisfied
+ * @user_data: the data to pass to callback function
  *
  * Asynchronously send credentials.
  *
@@ -590,12 +595,14 @@ g_unix_connection_receive_credentials (GUnixConnection      *connection,
     {
       if (nscm != 1)
         {
-          g_set_error_literal (error,
-                               G_IO_ERROR,
-                               G_IO_ERROR_FAILED,
-                               ngettext ("Expecting 1 control message, got %d",
-                                         "Expecting 1 control message, got %d",
-                                         nscm));
+          g_set_error (error,
+                       G_IO_ERROR,
+                       G_IO_ERROR_FAILED,
+                       g_dngettext (NULL,
+                                    "Expecting 1 control message, got %d",
+                                    "Expecting 1 control message, got %d",
+                                    nscm),
+                       nscm);
           goto out;
         }
 
@@ -683,8 +690,9 @@ receive_credentials_async_thread (GTask         *task,
  * g_unix_connection_receive_credentials_async:
  * @connection: A #GUnixConnection.
  * @cancellable: (nullable): optional #GCancellable object, %NULL to ignore.
- * @callback: (scope async): a #GAsyncReadyCallback to call when the request is satisfied
- * @user_data: (closure): the data to pass to callback function
+ * @callback: (scope async): a #GAsyncReadyCallback
+ *   to call when the request is satisfied
+ * @user_data: the data to pass to callback function
  *
  * Asynchronously receive credentials.
  *
